<?php
/**
 * ORM-Object to handle articles from Open Content
 */

namespace EwTools\Contenttypes;

use OcAPI;
use OcArticle;
use EwTools\Handler\OpenContent;
use EwTools\Support\Collection;
use WP_Post;

/**
 * Article
 *
 * @todo    Use handler for Open Content querybuilder
 * @link    http://infomaker.se
 * @package EwTools\Contenttypes
 * @since   EwTools\Contenttypes\Article 0.1
 * @property string $uuid          Unique id for article
 * @property string $permalink     Permalink to the article
 * @property int    $post_id       WordPress post id
 */
abstract class Article extends Contenttype {
    
    /**
     * Contains the type it represents in OC
     *
     * @var string
     */
    protected $contenttype = "Article";
    
    /**
     * Article constructor.
     *
     * @since 0.1
     * @param OcArticle $oc_article
     */
    public function __construct( OcArticle $oc_article ) {
        $this->fill( array_replace( $oc_article->get_all_properties(), [
            'uuid'      => $oc_article->get_value( 'uuid' ),
            'permalink' => $oc_article->get_permalink(),
            'post_id'   => $oc_article->get_post()->ID
        ] ) );
    }
    
    /**
     * Function to fetch articles and creating a collection of the requestad article
     * TODO: not finished Use OpenContent class for fetching articles
     *
     * @param string $query
     * @param int    $limit
     *
     * @since 0.1
     * @return Collection
     */
    public static function query( $query, $limit = null ) {
        $articles      = [];
        $search_result = OpenContent::search( [
            'query'      => $query,
            'use_raw'    => true,
            'sorting'    => 'Publiceringsdag',
            'properties' => [],
            'limit'      => $limit,
            'use_cache'  => true
        ] );
        
        if( is_array( $search_result ) ) {
            unset( $search_result[ 'facet' ] );
            unset( $search_result[ 'hits' ] );
            unset( $search_result[ 'duration' ] );
            
            $articles = array_map( function ( $article ) {
                return ( new static( $article ) );
            }, $search_result );
        }
        
        return new Collection( $articles );
    }
    
    /**
     * Function fetch an article and create an instance of requested class
     *
     * @param string $uuid
     *
     * @since 0.1
     * @return null|static
     */
    public static function create( $uuid ) {
        $article = OpenContent::getArticle( $uuid );
        if( $article instanceof OcArticle ) {
            
            return new static( $article );
        }
        
        return null;
    }
    
    /**
     * Function create an article from the post type
     *
     * @param WP_Post $post
     *
     * @since    0.1
     * @return null|static
     */
    public static function createFromPost( WP_Post $post ) {
        return static::create( get_post_meta( $post->ID, 'oc_uuid', true ) );
    }
}